package info.visone.nlpext.gui;

import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;

import edu.stanford.nlp.ling.HasWord;
import edu.stanford.nlp.parser.lexparser.LexicalizedParser;
import edu.stanford.nlp.parser.lexparser.LexicalizedParserQuery;
import edu.stanford.nlp.tagger.maxent.MaxentTagger;
import edu.stanford.nlp.trees.Tree;

import info.visone.base.Mediator;
import info.visone.base.Network;
import info.visone.ext.nlp.PennTreeAlgorithm;
import info.visone.gui.tasks.NetworkAlgorithmTask;
import info.visone.gui.window.VisoneWindow;

public class PennTreeTask extends NetworkAlgorithmTask<PennTreeAlgorithm> {
    private final LexicalizedParser parser;
    private final String sentence;

    public PennTreeTask(
            PennTreeAlgorithm algorithm,
            LexicalizedParser parser,
            String sentence,
            String eventName,
            Mediator mediator) {
        super(algorithm, mediator.createDummyNetwork(), false, eventName, false, mediator, null);
        this.parser = parser;
        this.sentence = sentence;
    }

    @Override
    protected void runAlgorithm(Network network) {

        final List<List<HasWord>> sentences = MaxentTagger.tokenizeText(new StringReader(sentence));
        if (sentences.isEmpty()) return;
        // it's supposed to be just one sentence so just concatenate
        // everything...
        final List<HasWord> words = new ArrayList<HasWord>();
        for (final List<HasWord> sent : sentences) words.addAll(sent);

        final LexicalizedParserQuery parserQuery = parser.parserQuery();
        if (parserQuery.parse(words)) {
            final Tree parsedTree = parserQuery.getBestParse();
            String s = parsedTree.pennString();
            getAlgorithm().setPennString(s);
            super.runAlgorithm(network);
        }
    }

    @Override
    protected void finishOnMainThread(Network network) {
        getMediator().setNetwork(network);
        VisoneWindow window = getMediator().getWindow();
        if (window != null) {
            window.doModeSelect();
            window.doFitContent(network);
        }
    }
}
